#!/usr/bin/env python
# encoding: utf-8
'''
@license: (C) Copyright 2021, Hey.
@author: Hey
@email: sanyuan.hy@alibaba-inc.com
@tel: 137****6540
@datetime: 2023/4/21 20:57
@project: LucaProtApp
@file: inference_time
@desc: xxxx
'''

import time
import os, csv, sys, torch
import argparse
sys.path.append(".")
sys.path.append("../")
sys.path.append("../algorithms")
try:
    from predict_one_sample import load_args, predict_multi_label, \
        predict_multi_class, predict_binary_class, load_model
except ImportError:
    from algorithms.predict_one_sample import load_args, predict_multi_label, \
        predict_multi_class, predict_binary_class, load_model

parser = argparse.ArgumentParser(description="Prediction")
parser.add_argument("--group_idx", default=-1, type=int, help="only running the group_idx not greater than this value(-1 for all)")
parser.add_argument("--data_path", default=None, type=str, required=True, help="data file path")
parser.add_argument("--truncation_seq_length", default=4096, type=int, help="truncation seq length")
parser.add_argument("--emb_dir", default=None, type=str, help="the structural embedding save dir. default: None")
parser.add_argument("--dataset_name", default="rdrp_40_extend", type=str, help="the dataset name for model building.")
parser.add_argument("--dataset_type", default="protein", type=str, help="the dataset type for model building.")
parser.add_argument("--task_type", default="binary_class", type=str, choices=["multi_label", "multi_class", "binary_class"], help="the task type for model buliding.")
parser.add_argument("--model_type", default="sefn", type=str, help="model type.")
parser.add_argument("--time_str", default="20230201140320", type=str, help="the running time string(yyyymmddHimiss) of model building.")
parser.add_argument("--step", default="100000", type=str, help="the training global step of model finalization.")
parser.add_argument("--threshold",  default=0.5, type=float, help="sigmoid threshold for binary-class or multi-label classification, None for multi-class classification, defualt: 0.5.")
parser.add_argument("--save_path", default=None, type=str, required=True, help="stats results save file path")
args = parser.parse_args()

model_dir = "../models/models/%s/%s/%s/%s/%s/%s" % (args.dataset_name, args.dataset_type, args.task_type,
                                                     args.model_type, args.time_str, args.step if args.step == "best" else "checkpoint-{}".format(args.step))
config_dir = "../models/logs/%s/%s/%s/%s/%s" % (args.dataset_name, args.dataset_type, args.task_type,
                                                args.model_type,  args.time_str)


group_names = [
    "300~500",
    "500~800",
    "800~1000",
    "1000~1500",
    "1500~2000",
    "2000~3000",
    "3000~5000",
    "5000~8000",
    "8000~10000",
    "10000+"
]


def load_data(data_filepath):
    selected_sequence_groups = [[], [], [], [], [], [], [], [], [], []]
    with open(data_filepath, "r") as rfp:
        reader = csv.reader(rfp)
        cnt = 0
        for row in reader:
            cnt += 1
            # 去掉表头
            if cnt == 1:
                continue
            # 每一行是一个样本
            seq_id, seq, label, group_idx = row[0], row[1], row[2], int(row[3])
            selected_sequence_groups[group_idx].append(row)
    # add one to load model(增加一个样本到第0组，也就是第0组第0个样本用来加载模型的，不计入时间）
    selected_sequence_groups[0].append(selected_sequence_groups[0][0])
    print("all num: ", [len(item) for item in selected_sequence_groups])
    return selected_sequence_groups


def inference_time_stats():
    # Step1: loading the model configuration
    config = load_args(config_dir)
    print("-" * 25 + "config:" + "-" * 25)
    print(config)
    if config:
        args.dataset_name = config["dataset_name"]
        args.dataset_type = config["dataset_type"]
        args.task_type = config["task_type"]
        args.model_type = config["model_type"]
        args.has_seq_encoder = config["has_seq_encoder"]
        args.has_struct_encoder = config["has_struct_encoder"]
        args.has_embedding_encoder = config["has_embedding_encoder"]
        args.subword = config["subword"]
        args.codes_file = config["codes_file"]
        args.input_mode = config["input_mode"]
        args.label_filepath = config["label_filepath"]
        if not os.path.exists(args.label_filepath):
            args.label_filepath = os.path.join(config_dir, "label.txt")
        args.output_dir = config["output_dir"]
        args.config_path = config["config_path"]

        args.do_lower_case = config["do_lower_case"]
        args.sigmoid = config["sigmoid"]
        args.loss_type = config["loss_type"]
        args.output_mode = config["output_mode"]
        args.device = torch.device("cuda") if torch.cuda.is_available() else torch.device("cpu")
        args.seq_vocab_path = config["seq_vocab_path"]
        args.seq_pooling_type = config["seq_pooling_type"]
        args.seq_max_length = config["seq_max_length"]
        args.struct_vocab_path = config["struct_vocab_path"]
        args.struct_max_length = config["struct_max_length"]
        args.struct_pooling_type = config["struct_pooling_type"]
        args.trunc_type = config["trunc_type"]
        args.no_position_embeddings = config["no_position_embeddings"]
        args.no_token_type_embeddings = config["no_token_type_embeddings"]
        args.cmap_type = config["cmap_type"]
        args.cmap_type = float(config["cmap_thresh"])
        args.embedding_input_size = config["embedding_input_size"]
        args.embedding_pooling_type = config["embedding_pooling_type"]
        args.embedding_max_length = config["embedding_max_length"]
        args.embedding_type = config["embedding_type"]
        if args.task_type in ["multi-label", "multi_label"]:
            args.sigmoid = True
        elif args.task_type in ["binary-class", "binary_class"]:
            args.sigmoid = True

    print("-" * 25 + "args:" + "-" * 25)
    print(args.__dict__.items())
    print("-" * 25 + "model_dir list:" + "-" * 25)
    print(os.listdir(model_dir))

    # Step2: loading the tokenizer and model
    config, subword, seq_tokenizer, struct_tokenizer, model, label_id_2_name, label_name_2_id = load_model(args, model_dir)
    if args.task_type in ["multi-label", "multi_label"]:
        predict_func = predict_multi_label
    elif args.task_type in ["binary-class", "binary_class"]:
        predict_func = predict_binary_class
    elif args.task_type in ["multi-class", "multi_class"]:
        predict_func = predict_multi_class
    else:
        raise Exception("Not Support Task Type: %s" % args.task_type)

    selected_sequence_groups = load_data(args.data_path)
    use_time = []
    for idx1, item in enumerate(selected_sequence_groups):
        if 0 <= args.group_idx < idx1:
            continue
        use_time.append([])
        for idx2, row in enumerate(item):
            protein_id, seq, label = row[0], row[1], row[2]
            row = [protein_id, seq]
            # Step 3: prediction
            begin = time.time()
            res = predict_func(args, label_id_2_name, seq_tokenizer, subword, struct_tokenizer, model, row)
            end = time.time()
            # idx = 0, the spend time includes the esmfold model loading time, so remove it
            if idx1 == 0 and idx2 == 0:
                continue
            use_time[-1].append([protein_id, seq, label, idx1, end - begin])
        if len(use_time[-1]) > 0:
            print("group name(seq length range): %s, num samples: %d, avg-time(s): %f, max-time(s): %f, min-time(s): %f"
                  % (group_names[idx1],
                     len(use_time[-1]),
                     sum([v[-1] for v in use_time[-1]])/len(use_time[-1]),
                     max([v[-1] for v in use_time[-1]]),
                     min([v[-1] for v in use_time[-1]]))
                  )
    with open(args.save_path, "w") as wfp:
        writer = csv.writer(wfp)
        writer.writerow(["protein_id", "seq", "label", "group_idx", "use_time(s)"])
        for item1 in use_time:
            for item2 in item1:
                writer.writerow(item2)


if __name__ == "__main__":
    inference_time_stats()
    '''
    cd LucaProtApp/algorithms/   
    export CUDA_VISIBLE_DEVICES=0
    python inference_time_stats_lucaprot_github.py \
        --data_path ../data/inference_time/inference_time_data_of_github.csv \
        --save_path ../data/inference_time/inference_time_stats_of_github.csv \
        --truncation_seq_length 4096 \
        --dataset_name rdrp_40_extend \
        --dataset_type protein \
        --task_type binary_class \
        --model_type sefn \
        --time_str 20230201140320 \
        --step 100000 \
        --threshold 0.5 \
        --group_idx -1
    
    simple:
    cd LucaProtApp/algorithms/    
    export CUDA_VISIBLE_DEVICES=0
    python inference_time_stats_lucaprot_github.py \
        --data_path ../data/inference_time/inference_time_data_of_github.csv \
        --save_path ../data/inference_time/inference_time_stats_of_github.csv \
        --group_idx -1
    '''











